## Analysis of accuracy and RT in all sound-color experiments

if (FALSE) {  # load once
  setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/010_soundCol_perceptual/analysis')
  source('soundCol_rules&labels.R')
  source('/home/allgoodguys/Documents/Soft_comp/R/ggplot_multiple.R')  # From http://www.cookbook-r.com/Graphs/Multiple_graphs_on_one_page_(ggplot2)/ 
  library(brms)
  library(ggplot2)
  all = read.csv('data_pooled.csv')
  probToOdds = function(x) x / (1 - x)
}

experiment = 'loudB'  # specify which experiment to analyze - see unique(all$experiment) for names of experiments

##############
# ANALYSIS
##############

## Load data
df = droplevels(all[all$experiment == experiment, ])
# aggregate(correct ~ target, df, mean)
# aggregate(correct ~ condition, df, mean)
# boxplot(log(responseTime) ~ condition, df[df$correct == TRUE, ])
# aggregate(responseTime ~ condition, df[df$correct == TRUE, ], mean)

df$pair = factor(df$pair, levels = c(rules[[experiment]]$label1, rules[[experiment]]$label2)) # to aggregate in the same order as in plots
# table(df$condition, df$pair)
# aggregate(correct ~ pair, df, mean)
# aggregate(correct ~ pair + subject, df, mean)

# aggregate(responseTime ~ pair, df, mean)
# aggregate(responseTime ~ pair + subject, df, mean)

## 
## ACCURACY
## 
if (nrow(df) < 1 | any(is.na(df$pair))) {
  print('PROBLEMS!!!')
} else {
  a_acc = as.data.frame(aggregate(correct ~ pair + subject, df, mean))
  a1_acc = reshape(a_acc, v.names = 'correct', idvar = 'subject', timevar = 'pair', direction = 'wide')
  # hist(a1_acc[, 2] - a1_acc[, 3])
  # table(a1_acc[, 2] > a1_acc[, 3])
  # colnames(a1_acc)
  
  modname_acc = paste0('models/mod_', experiment, '_acc.RDS')
  if (file.exists(modname_acc)) {
    mod1 = readRDS(modname_acc)
  } else {
    mod1 = brm(correct ~ pair + (pair|subject) + (1|target), data = df, family = 'bernoulli', iter = 4000, warmup = 500, cores = 2, chains = 2, prior = set_prior('normal(0, 3)'), control = list(adapt_delta = .95))
    saveRDS(mod1, modname_acc)
  }
  # summary(mod1)
  # plot(mod1)
  # pp_check(mod1)
  # brms::marginal_effects(mod1)
  
  ## contrasts per participant
  newdataPart_acc = expand.grid(pair = levels(df$pair),
                                subject = levels(df$subject))
  fitPart_acc = fitted(mod1, newdata = newdataPart_acc, re_formula = ~(pair|subject), summary = FALSE)
  colnames(fitPart_acc) = paste(newdataPart_acc$pair, newdataPart_acc$subject)
  contrastPart_acc = data.frame(
    subject = unique(newdataPart_acc$subject)
  )
  contrastPart_acc$upr = contrastPart_acc$lwr = contrastPart_acc$fit = NA
  for (i in 1:length(unique(newdataPart_acc$subject))) {
    j = 2 * i - 1
    # temp = probToOdds(fitPart_acc[, j + 1]) / probToOdds(fitPart_acc[, j])
    temp = fitPart_acc[, j + 1] - fitPart_acc[, j]
    contrastPart_acc[i, 2:4] = quantile(temp, probs = c(.5, .025, .975)) * 100
  }
  # observed
  # a1_acc$oddsEr = probToOdds(a1_acc[, 3]) / probToOdds(a1_acc[, 2])
  a1_acc$errProb = (a1_acc[, 3] - a1_acc[, 2]) * 100
  rangeObs = range(a1_acc$errProb[which(is.finite(a1_acc$errProb))])
  yl = c(min(c(contrastPart_acc$lwr, rangeObs), na.rm = T) - .05 * diff(rangeObs),
         max(c(contrastPart_acc$upr, rangeObs), na.rm = T) + .05 * diff(rangeObs))
  
  plot_acc = ggplot(contrastPart_acc, aes(x = subject, y = fit, ymin = lwr, ymax = upr)) +
    geom_pointrange(shape = 1) +
    geom_point(data = a1_acc, aes(x = subject, y = errProb), inherit.aes = FALSE, shape = 4, size = 4) +
    geom_hline(yintercept = 0, linetype = 2) +
    annotate('text', y = yl[2], x = 0,
             label = rules[[experiment]]$label_plot1, hjust = 1) +
    coord_flip() +
    scale_x_discrete(expand = c(0.05, 0)) +
    ylim(yl) +
    xlab('') + ylab('Difference in error rate, %') + 
    ggtitle(paste0('Accuracy: ', experiment)) +
    theme_bw() + theme(panel.grid = element_blank())
  
  
  ##
  ## RESPONSE TIME
  ## 
  # range(df$responseTime)  # ms
  # quantile(df$responseTime, probs = c(.9, .95, .99))
  # hist(df$responseTime)
  # hist(log(df$responseTime))
  
  dfRT = df[df$correct == TRUE, ]  # analyze RT only in trials with correct response!
  dfRT = dfRT[dfRT$responseTime < 5000, ]
  
  a_rt = as.data.frame(aggregate(responseTime ~ pair + subject, dfRT, mean))
  a1_rt = reshape(a_rt, v.names = 'responseTime', idvar = 'subject', timevar = 'pair', direction = 'wide')
  # hist(a1_rt[, 2] - a1_rt[, 3])
  # table(a1_rt[, 2] < a1_rt[, 3]) 
  # colnames(a1)
  
  modname_rt = paste0('models/mod_', experiment, '_RT.RDS')
  if (file.exists(modname_rt)) {
    mod2 = readRDS(modname_rt)
  } else {
    mod2 = brm(responseTime ~ pair + (pair|subject) + (1|target), data = dfRT, family = 'lognormal', iter = 4000, warmup = 1000, cores = 2, chains = 2, prior = set_prior('normal(0, 3)'), control = list(adapt_delta = .99))
    saveRDS(mod2, modname_rt)
  }
  # plot(mod2)
  # summary(mod2)
  # pp_check(mod2)
  # brms::marginal_effects(mod2)
  
  # RT per target * condition
  if (FALSE) {
    newdata3 = expand.grid(pair = levels(dfRT$pair), target = levels(dfRT$target))
    fit3 = fitted(mod2, newdata = newdata3, re_formula = ~(1|target))
    colnames(fit3) = c('fit', 'se', 'lwr', 'upr')
    fit3 = cbind(newdata3, fit3)
    fit3
  }
  
  ## contrasts per participant
  newdataPart_rt = expand.grid(pair = levels(dfRT$pair),
                               subject = levels(dfRT$subject))
  fitPart_rt = fitted(mod2, newdata = newdataPart_rt, re_formula = ~(pair|subject), summary = FALSE)
  colnames(fitPart_rt) = paste(newdataPart_rt$pair, newdataPart_rt$subject)
  contrastPart_rt = data.frame(
    subject = unique(newdataPart_rt$subject)
  )
  contrastPart_rt$upr = contrastPart_rt$lwr = contrastPart_rt$fit = NA
  for (i in 1:length(unique(newdataPart_rt$subject))) {
    j = 2 * i - 1
    temp = fitPart_rt[, j + 1] - fitPart_rt[, j]
    contrastPart_rt[i, 2:4] = round(quantile(temp, probs = c(.5, .025, .975)), 1)
  }
  # observed
  a1_rt$dif = a1_rt[, 3] - a1_rt[, 2]
  rangeObs_rt = range(a1_rt$dif[which(is.finite(a1_rt$dif))])
  yl_rt = c(min(c(contrastPart_rt$lwr, rangeObs_rt), na.rm = T) - .05 * diff(rangeObs_rt),
            max(c(contrastPart_rt$upr, rangeObs_rt), na.rm = T) + .05 * diff(rangeObs_rt))
  
  plot_rt = ggplot(contrastPart_rt, aes(x = subject, y = fit, ymin = lwr, ymax = upr)) +
    geom_pointrange(shape = 1) +
    geom_point(data = a1_rt, aes(x = subject, y = dif), inherit.aes = FALSE, shape = 4, size = 4) +
    geom_hline(yintercept = 1, linetype = 2) +
    annotate('text', y = yl_rt[1], x = 0,
             label = rules[[experiment]]$label_plot2, hjust = 0) +
    coord_flip() +
    scale_x_discrete(position = 'top', expand = c(0.05, 0)) +
    ylim(yl_rt) +
    xlab('') + ylab('Difference in RT, ms') + 
    ggtitle(paste0('RT: ', experiment)) +
    theme_bw() + theme(panel.grid = element_blank())
  
  plotname = paste0('pix/', experiment, '_ptcps.png')
  multiplot(plot_acc, plot_rt, cols = 2)
  ggsave(multiplot(plot_acc, plot_rt, cols = 2), filename = plotname, width = 25, height = 15, units = 'cm', dpi = 300)
}
