## Plotting accuracy and RT in all sound-color experiments

if (FALSE) {  # load once
  setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/010_soundCol_perceptual/analysis')
  source('soundCol_rules&labels.R')
  source('/home/allgoodguys/Documents/Soft_comp/R/ggplot_multiple.R')  # From http://www.cookbook-r.com/Graphs/Multiple_graphs_on_one_page_(ggplot2)/ 
  library(brms)
  library(ggplot2)
  all = read.csv('data_pooled.csv')
}

experiments = data.frame(
  loudLum = 'Loudness',
  loudA = 'Loudness',
  loudB = 'Loudness',
  loudSat = 'Loudness',
  pitchLum = 'Pitch',
  pitchA = 'Pitch',
  pitchB = 'Pitch',
  pitchSat = 'Pitch',
  F1Lum = 'F1',
  F1A = 'F1',
  F1B = 'F1',
  F2Lum = 'F2',
  F2A = 'F2',
  F2B = 'F2',
  specLum = 'Spectral\ncentroid',
  specA = 'Spectral\ncentroid',
  specB = 'Spectral\ncentroid',
  specSat = 'Spectral\ncentroid',
  trillLum = 'Trill',
  trillA = 'Trill',
  trillB = 'Trill',
  trillSat = 'Trill',
  stringsAsFactors = FALSE
)

df_plot = df_table = NULL
for (e in 1:ncol(experiments)) {
  experiment = colnames(experiments)[e]
  all_exp = droplevels(all[all$experiment == experiment, ])
  all_exp$pair = factor(all_exp$pair, levels = c(rules[[experiment]]$label1, rules[[experiment]]$label2))
  
  # Accuracy
  mod_acc = readRDS(paste0('models/mod_', experiment, '_acc.RDS'))
  pair = data.frame(pair = c(rules[[experiment]]$label1, rules[[experiment]]$label2))
  fit_acc_contrast = fitted(mod_acc, newdata = pair, re_formula = NA, summary = FALSE)
  # Note the order: 1 - 2, ie incongruent - congruent (expect more errors and longer RT)
  contrast_err_percent = quantile(((100 - fit_acc_contrast[, 1]) - (100 - fit_acc_contrast[, 2])) * 100, probs = c(.5, .025, .975))  # contrast in % for errors, not accuracy
  fit_acc_table = fitted(mod_acc, newdata = pair, re_formula = NA, robust = TRUE) * 100
  
  # RT
  mod_rt = readRDS(paste0('models/mod_', experiment, '_RT.RDS'))
  fit_rt_contrast = fitted(mod_rt, newdata = pair, re_formula = NA, summary = FALSE)
  contrast_rt = quantile((fit_rt_contrast[, 1] - fit_rt_contrast[, 2]), probs = c(.5, .025, .975))
  # Reverse the contrast direction for RT for better plotting
  contrast_rt_rev = quantile((fit_rt_contrast[, 2] - fit_rt_contrast[, 1]), probs = c(.5, .025, .975))
  fit_rt_table = fitted(mod_rt, newdata = pair, re_formula = NA, robust = TRUE)
  
  df_plot_exp = data.frame(
    experiment = rep(as.character(experiments[[experiment]]), 2),  # full name of acoustic condition
    measure = c('Accuracy', 'RT'),
    label1 = rep(rules[[experiment]]$label_plot1, 2),
    label2 = rep(rules[[experiment]]$label_plot2, 2),
    fit = c(as.numeric(contrast_err_percent[1]), as.numeric(contrast_rt_rev[1])),
    lwr = c(as.numeric(contrast_err_percent[2]), as.numeric(contrast_rt_rev[2])),
    upr = c(as.numeric(contrast_err_percent[3]), as.numeric(contrast_rt_rev[3]))
  )
  
  df_table_exp = data.frame(
    experiment = rep(as.character(experiments[[experiment]]), 2),
    rule = c(rules[[experiment]]$label_plot1, rules[[experiment]]$label_plot2),
    errorRate_obs = round(100 - aggregate(correct ~ pair, all_exp, mean)[, 2] * 100, 1),
    errorRate = round(100 - as.numeric(fit_acc_table[, 1]), 1),
    contrast_acc = c(soundgen:::reportCI(contrast_err_percent, digits = 1), ''),
    RT_obs = round(aggregate(responseTime ~ pair, all_exp, mean)[, 2], 0),
    RT = round(as.numeric(fit_rt_table[, 1]), 0),
    contrast_rt = c(soundgen:::reportCI(contrast_rt, digits = 0), '')
  )
  
  # Visual group (L / a / b)
  if (grepl(x = experiment, pattern = 'Lum', fixed = TRUE)) {
    df_plot_exp$group = 'L'
  } else if (grepl(x = experiment, pattern = 'A', fixed = TRUE)) {
    df_plot_exp$group = 'a'
  } else if (grepl(x = experiment, pattern = 'B', fixed = TRUE)) {
    df_plot_exp$group = 'b'
  } else if (grepl(x = experiment, pattern = 'Sat', fixed = TRUE)) {
    df_plot_exp$group = 'Sat'
  } 
  
  if (is.null(df_plot)) {
    df_plot = df_plot_exp
  } else {
    df_plot = rbind(df_plot, df_plot_exp)
  }
  
  if (is.null(df_table)) {
    df_table = df_table_exp
  } else {
    df_table = rbind(df_table, df_table_exp)
  }
}

df_plot$label_final = ifelse(
  df_plot$lwr * df_plot$upr < 0,  # 95% CI overlaps with 0
  '',
  ifelse(df_plot$fit > 0, as.character(df_plot$label1), as.character(df_plot$label2))
)
df_plot$experiment = factor(df_plot$experiment, levels = rev(unique(df_plot$experiment)))
df_plot$group = factor(df_plot$group, levels = rev(c('L', 'a', 'b', 'Sat')))


## Extract observed contrasts (per condition) in accuracy and RT per participant in each experiment
observedAcc = observedRT = NULL
for (e in 1:ncol(experiments)) {
  experiment = colnames(experiments)[e]
  df_exp = droplevels(all[all$experiment == experiment, ])
  
  # Accuracy
  a_acc = as.data.frame(aggregate(correct ~ pair + subject, df_exp, mean))
  a1_acc = reshape(a_acc, v.names = 'correct', idvar = 'subject', timevar = 'pair', direction = 'wide')
  # contrast for errors, therefore 2 - 1, not 1 - 2
  a1_acc$contrast = (a1_acc[, paste0('correct.', rules[[experiment]]$label2)] - 
                       a1_acc[, paste0('correct.', rules[[experiment]]$label1)]) * 100
  a1_acc$experiment = experiment
  a1_acc = a1_acc[, c('experiment', 'subject', 'contrast')]
  if (is.null(observedAcc)) {
    observedAcc = a1_acc
  } else {
    observedAcc = rbind(observedAcc, a1_acc)
  }
  
  # RT
  a_rt = as.data.frame(aggregate(responseTime ~ pair + subject, df_exp, mean))
  a1_rt = reshape(a_rt, v.names = 'responseTime', idvar = 'subject', timevar = 'pair', direction = 'wide')
  # reversed for plotting, therefore 2 - 1, not 1 - 2
  a1_rt$contrast = (a1_rt[, paste0('responseTime.', rules[[experiment]]$label2)] - 
                      a1_rt[, paste0('responseTime.', rules[[experiment]]$label1)])
  a1_rt$experiment = experiment
  a1_rt = a1_rt[, c('experiment', 'subject', 'contrast')]
  if (is.null(observedRT)) {
    observedRT = a1_rt
  } else {
    observedRT = rbind(observedRT, a1_rt)
  }
}

for (i in 1:nrow(observedAcc)) {
  e = as.character(observedAcc$experiment[i])
  if (grepl(x = e, pattern = 'Lum', fixed = TRUE)) {
    observedAcc$group[i] = 'L'
  } else if (grepl(x = e, pattern = 'A', fixed = TRUE)) {
    observedAcc$group[i] = 'a'
  } else if (grepl(x = e, pattern = 'B', fixed = TRUE)) {
    observedAcc$group[i] = 'b'
  } else if (grepl(x = e, pattern = 'Sat', fixed = TRUE)) {
    observedAcc$group[i] = 'Sat'
  }
}
observedAcc$experiment = as.character(experiments[as.character(observedAcc$experiment)])
observedAcc$group = factor(observedAcc$group, levels = levels(df_plot$group))

for (i in 1:nrow(observedRT)) {
  e = as.character(observedRT$experiment[i])
  if (grepl(x = e, pattern = 'Lum', fixed = TRUE)) {
    observedRT$group[i] = 'L'
  } else if (grepl(x = e, pattern = 'A', fixed = TRUE)) {
    observedRT$group[i] = 'a'
  } else if (grepl(x = e, pattern = 'B', fixed = TRUE)) {
    observedRT$group[i] = 'b'
  } else if (grepl(x = e, pattern = 'Sat', fixed = TRUE)) {
    observedRT$group[i] = 'Sat'
  }
}
observedRT$experiment = as.character(experiments[as.character(observedRT$experiment)])
observedRT$group = factor(observedRT$group, levels = levels(df_plot$group))

## Prepare separate half-plots for accuracy and RT
# Accuracy
df_plot_acc = df_plot[df_plot$measure == 'Accuracy', ]

plot_acc = ggplot(df_plot_acc, aes(x = experiment, y = fit, ymin = lwr, ymax = upr, col = group, shape = group)) +
  geom_point(position = position_dodge(.7), size = 0) +  # only for correct order of factor levels in geom_violin
  geom_violin(data = observedAcc, aes(x = experiment, y = contrast, group = interaction(experiment, group)), inherit.aes = FALSE, col = 'white', fill = 'gray50', alpha = .25, show.legend = FALSE, position = position_dodge(.7)) +
  geom_pointrange(position = position_dodge(.7)) +
  geom_text(aes(label = label_final), position = position_dodge(.7), size = 2.5, col = 'black', vjust = -1, hjust = .5, show.legend = FALSE) +
  scale_color_manual(name = '', breaks = rev(levels(df_plot_acc$group)), values = rev(c('black', 'green', 'blue', 'red'))) +
  scale_shape_manual(name='', breaks = rev(levels(df_plot_acc$group)), values = c(18, 15:17)) +
  ylab('Difference in error rates, %') + xlab('') +
  geom_hline(yintercept = 0, linetype = 2) +
  ggtitle('A. Accuracy') +
  annotate('text', x = 6.5, y = 15, label = 'More errors', hjust = 1, size = 3) +
  coord_flip() +
  theme_bw() +
  theme(panel.grid = element_blank(), 
        axis.title.x = element_text(size = 10))

# RT
df_plot_rt = df_plot[df_plot$measure == 'RT', ]
plot_rt = ggplot(df_plot_rt, aes(x = experiment, y = fit, ymin = lwr, ymax = upr, col = group, shape = group)) +
  geom_point(position = position_dodge(.7), size = 0) +
  geom_violin(data = observedRT, aes(x = experiment, y = contrast, group = interaction(experiment, group)), inherit.aes = FALSE, col = 'white', fill = 'gray50', alpha = .25, show.legend = FALSE, position = position_dodge(.7)) +
  geom_pointrange(position = position_dodge(.7)) +
  geom_text(aes(label = label_final), position = position_dodge(.7), size = 2.5, col = 'black', vjust = -1, hjust = .5, show.legend = FALSE) +
  scale_color_manual(name = '', breaks = rev(levels(df_plot_rt$group)), values = rev(c('black', 'green', 'blue', 'red'))) +
  scale_shape_manual(name='', breaks = rev(levels(df_plot_rt$group)), values = c(18, 15:17)) +
  ylab('Difference in response times, ms') + xlab('') +
  geom_hline(yintercept = 0, linetype = 2) +
  ggtitle('B. Response time') +
  annotate('text', x = 6.5, y = -400, label = 'Faster response', hjust = 0, size = 3) +
  coord_flip() +
  scale_x_discrete(position = 'top') +  # extreme values for some subjects
  ylim(c(-400, 250)) +   # extreme values for some subjects
  theme_bw() +
  theme(panel.grid = element_blank(), 
        axis.title.x = element_text(size = 10))

# Save common plot
multiplot(plot_acc, plot_rt, cols = 2)
# ggsave(multiplot(plot_acc, plot_rt, cols = 2), filename = 'pix/00_plot_all.png', width = 20, height = 20, units = 'cm', dpi = 300)

# write.csv(df_table, '~/Downloads/table.csv')
